<?if (!defined('B_PROLOG_INCLUDED') || B_PROLOG_INCLUDED!==true) die();
/** @var CBitrixComponent $this */
/** @var array $arParams */
/** @var array $arResult */
/** @var string $componentPath */
/** @var string $componentName */
/** @var string $componentTemplate */
/** @global CDatabase $DB */
/** @global CUser $USER */
/** @global CMain $APPLICATION */
/** @global CCacheManager $CACHE_MANAGER */

class LogisticManagerStat {

  // 
  private $component, $app, $db, $user;

  // Bitrix-style 
  private $arParams = array(), $arResult = array(
      'CLOSED_REQUEST' => 0,
      'ITEMS' => array()
      //  : 'ITEMS'  
      //                  CLOSED_REQUEST
      //                  ALL_TU
      //                  DRIVER_INTIME
      //                  DRIVER_OUTTIME
      //                  DRIVER_REVERSE
    ),
    $arCompanyIds = array(), $arTenders = array(), $arBids = array();

  //  
  private $userID, $AD_CACHE_ID, $iAmTransporter;

  public function __construct(CBitrixComponent $component) {
    $this->component = $component;
    $this->component->arResult =& $this->arResult;
    $this->component->arParams =& $this->arParams;

    global $APPLICATION, $DB, $USER;
    $this->app = $APPLICATION;
    $this->db = $DB;
    $this->user = $USER;
    $this->userID = $this->user->GetID();

    CModule::IncludeModule('iblock');

    $arParams =& $this->arParams;
    $arResult =& $this->arResult;

    $arParams['CACHE_TYPE'] = 'A';
    $arParams['CACHE_TIME'] = 300;

    $arResult['AJAX_STAT_ALL_URL']    = '?ajax=Y' . (isset($_REQUEST['clear_cache']) ? '&clear_cache=' . $_REQUEST['clear_cache'] : '');
    $arResult['AJAX_OUTTIME_URL']     = '?ajax=Y&outtime=Y';


    $this->iAmTransporter = isUserLogTransporter($this->userID);
    $this->iAmAdmin       = !$this->iAmTransporter;
  }

  //    ''
  //   - 
  public function run() {
    $this->initFilters();
    $this->tryViewCache();
    $this->getCompanies();
    if (!isset($_REQUEST['ajax'])) {
      $this->viewHtml();
      return;
    }

    $this->app->RestartBuffer();

    $this->getBids();
    $this->getTenders();
    $this->calc();
    $this->getNowKpi();

    if($this->iAmTransporter) $this->getTotalClosedReq();

    $this->arResult['ITEMS'] = array_filter($this->arResult['ITEMS'], function($tender){
      return isset($tender['NAME'], $tender['CLOSED_REQUEST']) && !empty($tender['NAME']);
    });
    usort($this->arResult['ITEMS'], array('self', 'sortItems'));

    $this->viewAjax();
  }

  //     -   'tender.list'
  //   
  private $tenderBidMap = array(), $bidCompanyMap = array();
  private function getBids() {
    if(count($this->arCompanyIds) == 0) return;

    $arFilter = array(
      'IBLOCK_ID' => LOGISTIC_TRANSPORTERS_RESPONSE_IBID,
      'PROPERTY_USER_PARENT' => $this->arCompanyIds,
      // 'ACTIVE'  => 'Y' // <=  
    );

    $res = CIBlockElement::GetList(false, $arFilter, false, false,
      array('ID', 'IBLOCK_ID', 'PROPERTY_TENDER_PARENT', 'PROPERTY_USER_PARENT',
        'PROPERTY_CONFIRMATION', 'PROPERTY_BET', 'PROPERTY_DRIVER_ON_DATE'));


    while ($bid = $res->Fetch()) {
      if(!isset($bid['PROPERTY_TENDER_PARENT_VALUE'])) continue;
      $this->arBids[$bid['ID']] = $bid;

      //  
      $this->tenderBidMap[$bid['PROPERTY_TENDER_PARENT_VALUE']][] =
        array('ID' => $bid['ID'], 'USER' => $bid['PROPERTY_USER_PARENT_VALUE']);

      $this->bidCompanyMap[$bid['ID']] = $bid['PROPERTY_USER_PARENT_VALUE'];

      $transporter =& $this->arResult['ITEMS'][$bid['PROPERTY_USER_PARENT_VALUE']];


      //   ()

      if($bid['PROPERTY_CONFIRMATION_ENUM_ID'] == LOGISTIC_REVERSE) {
        ++$transporter['DRIVER_REVERSE'];
      }

    }
  }

  //        ,     ,    
  private function calc() {
    foreach($this->arBids as $bid) {

      $tender =& $this->arTenders[$bid['PROPERTY_TENDER_PARENT_VALUE']];
      if(!isset($tender)) continue;
      if(
        !in_array($tender['PROPERTY_TENDER_STATUS_ENUM_ID'], array(LOGISTIC_STATUS_OPEN, LOGISTIC_STATUS_CLOSE))
        && $tender['PROPERTY_CHOICE_REQUEST_VALUE'] == $bid['ID']
        ) {
          $transporter =& $this->arResult['ITEMS'][$bid['PROPERTY_USER_PARENT_VALUE']];
          $transporter['TU'] += $bid['PROPERTY_BET_VALUE'];
          // BET - 
          $this->arResult['ALL_TU'] += $bid['PROPERTY_BET_VALUE'];
      }
    }

    //  ,   
    $datetime_fact_now = date_create(date('d.m.Y H:i:s'));
    $datetime_fact_now->modify('+3 hour');

    foreach($this->arTenders as $tender)  {

        $datetime_to = date_create($tender['PROPERTY_LOADING_DATE_VALUE'] . " " . $tender['PROPERTY_LOADING_TIME_VALUE']);
        if (!$datetime_to) {
          continue;
        }

        $datetime_to->modify('+3 hour');

        $bid =& $this->arBids[$tender['PROPERTY_CHOICE_REQUEST_VALUE']];
        $transporter =& $this->arResult['ITEMS'][$bid['PROPERTY_USER_PARENT_VALUE']];

        if (!empty($bid['PROPERTY_DRIVER_ON_DATE_VALUE']))
        {
          $datetime_fact = date_create($bid['PROPERTY_DRIVER_ON_DATE_VALUE']);
        }
        else
        {
          $datetime_fact = clone $datetime_fact_now;
        }

        // TEST CASE
        /*
        if($bid['PROPERTY_USER_PARENT_VALUE'] == 353565 && $bid['ID'] == 1710025) {
        }
        */

        if ($datetime_to > $datetime_fact)
        {
          ++$this->arResult['DRIVER_INTIME'];
          ++$transporter['DRIVER_INTIME'];
        }
        else
        {
          ++$this->arResult['DRIVER_OUTTIME'];
          ++$transporter['DRIVER_OUTTIME'];
          $transporter['DRIVER_OUTTIME_ARRAY'][] = Array('tender' => $tender, 'trans' => $bid);
        }
    }
  }

  private function getTenders() {

    $arSelect = Array('IBLOCK_ID', 'ID', 'PROPERTY_CHOICE_REQUEST', 'PROPERTY_TENDER_STATUS', 'PROPERTY_LOADING_DATE', 'PROPERTY_LOADING_TIME');

    $arFilter = array(
      'IBLOCK_ID' => LOGISTIC_TENDER_REQUEST_IBID,
      'ACTIVE' => 'Y'
      // TODO???:    ,      -  ?
      // 'PROPERTY_CHOICE_REQUEST'
    );

    //    
    if (!empty($_SESSION['stat_filter_from']) && !empty($_SESSION['stat_filter_to'])) {
      $arFilter['><PROPERTY_LOADING_DATE'] = array(
        date_create($_SESSION['stat_filter_from'])->format('Y-m-d') . ' 00:00:00',
        date_create($_SESSION['stat_filter_to'])->format('Y-m-d') . ' 23:59:59'
      );
    }



    switch($_SESSION['stat_filter_type']) {
      case 2: //  [+]
        $arFilter[] = array( // [+]
          'LOGIC' => 'OR',
          array('!=PROPERTY_TENDER_STATUS'=>LOGISTIC_STATUS_OPEN, 'PROPERTY_IS_CITY_TRANS' => array('MG')),
          array('!=PROPERTY_TENDER_STATUS'=>LOGISTIC_STATUS_OPEN, 'PROPERTY_IS_CITY_TRANS' => false, 'PROPERTY_IS_ASSIGNED' => false)
        );
        break;
      case 3: //  [+]
        $arFilter['PROPERTY_IS_CITY_TRANS'] = false;
        $arFilter['!=PROPERTY_IS_ASSIGNED'] = false;
        break;
      case 4: //   [+]
        $arFilter["PROPERTY_IS_CITY_TRANS"] = "Y";
        break;
      case 5: //  [?]  "3",     
        $arFilter['PROPERTY_IS_CITY_TRANS'] = false;
        $arFilter['PROPERTY_IS_ASSIGNED'] = false;
        break;
      default: // no filter
    }


    $this->arFilterTransTemp = $arFilter;
    $res = CIBlockElement::GetList(Array('ID' => 'ASC'), $arFilter, false, false, $arSelect);
    while($tender = $res->Fetch()) {
      //
      if(empty($tender['PROPERTY_CHOICE_REQUEST_VALUE'])) continue;
      $this->arTenders[$tender['ID']] = $tender;

      $status =& $tender['PROPERTY_TENDER_STATUS_ENUM_ID'];
      $transporterId =& $this->bidCompanyMap[$tender['PROPERTY_CHOICE_REQUEST_VALUE']];



      if($status != LOGISTIC_STATUS_OPEN && $status != LOGISTIC_STATUS_CLOSE) {
        //    
        ++$this->arResult['CLOSED_REQUEST'];
        //     
        if(isset($transporterId)) {
          ++$this->arResult['ITEMS'][$transporterId]['CLOSED_REQUEST'];
        }
      }

    }


  }

  //  
  private function getCompanies() {
    $arFilterCompany = array('IBLOCK_ID' => LOGISTIC_COMPANY_IBID, 'ACTIVE' => 'Y');

    // KPI     (       ),
    //     
    if ($this->iAmAdmin) {
      $arFilterCompany['!PROPERTY_CARGO_TYPE'] = LOGISTIC_CARGO_TYPE_TRAIN;
    }

    if ($_SESSION['stat_filter_type'] == 5) {
      unset($arFilterCompany['!PROPERTY_CARGO_TYPE']);
      $arFilterCompany['PROPERTY_CARGO_TYPE'] = LOGISTIC_CARGO_TYPE_TRAIN;
    }

    if (isset($_SESSION['stat_filter_company']))
      $arFilterCompany['ID'] = $_SESSION['stat_filter_company'];

    $res = CIBlockElement::GetList(Array('PROPERTY_TRANS_KPI' => 'desc'), $arFilterCompany, array('IBLOCK_ID', 'NAME', 'ID', 'PROPERTY_TRANS_KPI'));

    $arResult =& $this->arResult;
    $arCompanyIds =& $this->arCompanyIds;

    while ($ob = $res->GetNextElement()) {
      $arFields = $ob->GetFields();
      $xCompanyIds[] = $arFields['ID'];

      $arResult['COMPANY_HTML'] .= IWebAUX::OptionWrapper($arFields['ID'], $_SESSION['stat_filter_company'], $arFields['NAME'], true);

      $company =& $arResult['ITEMS'][$arFields['ID']];

      if (!empty($_SESSION['stat_filter_company']) && $arFields['ID'] == $_SESSION['stat_filter_company']) {
        $company['NAME'] = $arFields['NAME'];
        $company['TRANS_KPI'] = $arFields['PROPERTY_TRANS_KPI_VALUE'];
        $arCompanyIds[] = $arFields['ID'];
      }

      if (empty($_SESSION['stat_filter_company'])) {
        $company['NAME'] = $arFields['NAME'];
        $company['TRANS_KPI'] = $arFields['PROPERTY_TRANS_KPI_VALUE'];
        $arCompanyIds[] = $arFields['ID'];
      }
    }
  }

  private function getNowKpi() {
    $filter_from = date_create($_SESSION["stat_filter_from"]);
    $filter_to = date_create($_SESSION["stat_filter_to"]);
    $result = file_get_contents("http://" . $_SERVER['SERVER_NAME']
      . "/logistic/scripts/kpi_calculate.php?type=curl&date_from=" . $filter_from->format('d.m.Y')
      . "&date_to=" . $filter_to->format('d.m.Y'));
    if (!empty($result)) {
      $decode_result = json_decode($result, true);
      foreach ($decode_result as $key => $value) {
        $transporter =& $this->arResult['ITEMS'][$key];
        if (isset($transporter))
          $transporter['NOW_KPI'] = $value;
      }
    }
  }

  private function sortItems($a, $b) {
    if ($a['NOW_KPI'] > $b['NOW_KPI']) return -1;
    if ($a['NOW_KPI'] < $b['NOW_KPI']) return 1;
    if ($a['CLOSED_REQUEST'] == $b['CLOSED_REQUEST']) return 0; // bugfix in prev algo
    return $a['CLOSED_REQUEST'] > $b['CLOSED_REQUEST'] ? -1 : 1;
  }

  private $arFilterTransTemp;
  private function getTotalClosedReq() {
    $arFilterTransTemp =& $this->arFilterTransTemp;
    $arFilterTransTemp['!=PROPERTY_TENDER_STATUS'] = array(LOGISTIC_STATUS_OPEN, LOGISTIC_STATUS_CLOSE);
    $arFilterTransTemp['!=PROPERTY_CHOICE_REQUEST'] = false;
    $res = CIBlockElement::GetList(false, $arFilterTransTemp, array('IBLOCK_ID'));
    if ($el = $res->Fetch())
      $this->arResult['CLOSED_REQUEST'] = $el['CNT'];
  }

  ///////////////////////////// 


  private function initFilters() {

    //   '' 
    if (isset($_REQUEST['stat_filter_submit'])) {
      $this->setFilters();
    } elseif(isset($_REQUEST['stat_filter_clear'])) {
      //   '' 
      $this->resetFilters();
    }

    //   ,     '  '
    if ($this->iAmTransporter) {
      $_SESSION['stat_filter_company'] = $_SESSION['LOGISTIC_TRANSPORTER_COMPANY'];
      if (empty($_SESSION['stat_filter_company'])) $_SESSION['stat_filter_company'] = 0;
    }
    $this->AD_CACHE_ID = array(
      $this->userID,
      $_SESSION['stat_filter_company'],
      $_SESSION['stat_filter_type'],
      $_SESSION['stat_filter_from'],
      $_SESSION['stat_filter_to']);
  }

  private function setFilters() {
    $_SESSION['stat_filter_company']  = htmlspecialchars($_REQUEST['company']);
    $_SESSION['stat_filter_type']     = htmlspecialchars($_REQUEST['stat_type']);
    $_SESSION['stat_filter_from']     = htmlspecialchars($_REQUEST['stat_filter_from']);
    $_SESSION['stat_filter_to']       = htmlspecialchars($_REQUEST['stat_filter_to']);

    if(!$_SESSION['stat_filter_from']) {
      $_SESSION['stat_filter_from'] = '01.01.2015';
    }

    //   
    if ($this->iAmAdmin) {
      if ($_REQUEST['company'] != 'all') {
        $_SESSION['stat_filter_company'] = htmlspecialchars($_REQUEST['company']);
      } else {
        unset($_SESSION['stat_filter_company']);
      }
      if ($_REQUEST['stat_type'] != 'all') {
        $_SESSION['stat_filter_type'] = htmlspecialchars($_REQUEST['stat_type']);
      } else {
        unset($_SESSION['stat_filter_type']);
      }
    }
  }

  private function resetFilters() {
    unset($_SESSION['stat_filter_company'],
          $_SESSION['stat_filter_type'],
          $_SESSION['stat_filter_from'],
          $_SESSION['stat_filter_to']
    );
  }



  // 
  private function tryViewCache() {
    if (isset($_REQUEST['ajax']) && !$this->component->startResultCache(false, $this->AD_CACHE_ID)) {
      $this->app->RestartBuffer();
      $this->component->IncludeComponentTemplate(isset($_REQUEST['outtime']) ? 'outtime' : 'total');
      die;
    } else {
      $this->component->abortResultCache();
    }
  }

  private function viewAjax() {
    if ($this->component->startResultCache(false, $this->AD_CACHE_ID)) {
      $this->component->IncludeComponentTemplate(isset($_REQUEST['outtime']) ? 'outtime' : 'total');
      $this->component->endResultCache();
      die;
    }
  }

  private function viewHtml() {
    $this->component->IncludeComponentTemplate();
  }

}

$CBitrixComponent = new LogisticManagerStat($this);
$CBitrixComponent->run();
